@@ -37,6 +37,8 @@
  * (may need to add more WARN_PHP_xxx codes...)
  */
 
+char cvsroot_php_cxx[] = "$Id: php.cxx 13105 2012-05-23 02:58:24Z olly $";
+
 #include "swigmod.h"
 
 #include <ctype.h>
@@ -352,9 +354,7 @@ public:
       Printf(f_directors_h, "#ifndef SWIG_%s_WRAP_H_\n", cap_module);
       Printf(f_directors_h, "#define SWIG_%s_WRAP_H_\n\n", cap_module);
 
-      String *filename = Swig_file_filename(outfile_h);
-      Printf(f_directors, "\n#include \"%s\"\n\n", filename);
-      Delete(filename);
+      Printf(f_directors, "\n#include \"%s\"\n\n", Swig_file_filename(outfile_h));
     }
 
     /* PHP module file */
@@ -416,7 +416,6 @@ public:
     Append(s_header, "static void SWIG_FAIL() __attribute__ ((__noreturn__));\n");
     Append(s_header, "#endif\n\n");
     Append(s_header, "static void SWIG_FAIL() {\n");
-    Append(s_header, "    TSRMLS_FETCH();\n");
     Append(s_header, "    zend_error(SWIG_ErrorCode(), \"%s\", SWIG_ErrorMsg());\n");
     // zend_error() should never return with the parameters we pass, but if it
     // does, we really don't want to let SWIG_FAIL() return.  This also avoids
@@ -616,7 +615,7 @@ public:
 
     Printf(f_h, "#endif /* PHP_%s_H */\n", cap_module);
 
-    Delete(f_h);
+    Close(f_h);
 
     String *type_table = NewStringEmpty();
     SwigType_emit_type_table(f_runtime, type_table);
@@ -631,7 +630,7 @@ public:
       Dump(f_directors_h, f_runtime_h);
       Printf(f_runtime_h, "\n");
       Printf(f_runtime_h, "#endif\n");
-      Delete(f_runtime_h);
+      Close(f_runtime_h);
     }
 
     Printf(s_header, "/* end header section */\n");
@@ -655,6 +654,7 @@ public:
     Delete(s_vdecl);
     Delete(all_cs_entry);
     Delete(s_entry);
+    Close(f_begin);
     Delete(f_runtime);
     Delete(f_begin);
 
@@ -667,7 +667,7 @@ public:
       s_fakeoowrappers = NULL;
     }
     Printf(f_phpcode, "%s\n?>\n", s_phpclasses);
-    Delete(f_phpcode);
+    Close(f_phpcode);
 
     return SWIG_OK;
   }
@@ -787,6 +787,7 @@ public:
     }
 
     f = NewWrapper();
+    numopt = 0;
 
     String *outarg = NewStringEmpty();
     String *cleanup = NewStringEmpty();
@@ -1089,6 +1090,9 @@ public:
       int min_num_of_arguments = emit_num_required(l);
       int max_num_of_arguments = emit_num_arguments(l);
 
+      // For a function with default arguments, we end up with the fullest
+      // parmlist in full_parmlist.
+      ParmList *full_parmlist = l;
       Hash *ret_types = NewHash();
       Setattr(ret_types, d, d);
 
@@ -1121,6 +1125,7 @@ public:
 
 	  if (num_arguments > max_num_of_arguments) {
 	    max_num_of_arguments = num_arguments;
+	    full_parmlist = l2;
 	  }
 	  o = Getattr(o, "sym:nextSibling");
 	}
@@ -1171,6 +1176,11 @@ public:
 	if (!o) {
 	  // This "overloaded method" is really just one with default args.
 	  really_overloaded = false;
+	  if (l != full_parmlist) {
+	    l = full_parmlist;
+	    if (wrapperType == memberfn)
+	      l = nextSibling(l);
+	  }
 	}
       }
 
@@ -1571,8 +1581,7 @@ public:
 	  while (i.item) {
 	    Node *j = firstChild(i.item);
 	    while (j) {
-	      String *jname = Getattr(j, "name");
-	      if (!jname || Strcmp(jname, Getattr(n, "name")) != 0) {
+	      if (Strcmp(Getattr(j, "name"), Getattr(n, "name")) != 0) {
 		j = nextSibling(j);
 		continue;
 	      }
@@ -1693,9 +1702,9 @@ public:
 	  Printf(output, "\t\t$r=%s;\n", invoke);
 	if (Len(ret_types) == 1) {
 	  /* If d is abstract we can't create a new wrapper type d. */
-	  Node *d_class = classLookup(d);
+	  Node * d_class = classLookup(d);
 	  int is_abstract = 0;
-	  if (Getattr(d_class, "abstracts")) {
+	  if (Getattr(d_class, "abstract")) {
 	    is_abstract = 1;
 	  }
 	  if (newobject || !is_abstract) {
@@ -1735,6 +1744,7 @@ public:
 	    if (!class_node) {
 	      /* This is needed when we're returning a pointer to a type
 	       * rather than returning the type by value or reference. */
+	      class_node = current_class;
 	      Delete(mangled);
 	      mangled = NewString(SwigType_manglestr(ret_type));
 	      class_node = Getattr(zend_types, mangled);
@@ -2014,7 +2024,7 @@ done:
 	base.item = NULL;
       }
 
-      if (Getattr(n, "abstracts") && !GetFlag(n, "feature:notabstract")) {
+      if (Getattr(n, "abstract") && !GetFlag(n, "feature:notabstract")) {
 	Printf(s_phpclasses, "abstract ");
       }
 
@@ -2321,7 +2331,6 @@ done:
     Printf(f->code, "}\n");
 
     Wrapper_print(f, s_wrappers);
-    DelWrapper(f);
 
     return SWIG_OK;
   }
@@ -2406,17 +2415,18 @@ done:
   int classDirectorMethod(Node *n, Node *parent, String *super) {
     int is_void = 0;
     int is_pointer = 0;
-    String *decl = Getattr(n, "decl");
-    String *returntype = Getattr(n, "type");
-    String *name = Getattr(n, "name");
-    String *classname = Getattr(parent, "sym:name");
+    String *decl;
+    String *type;
+    String *name;
+    String *classname;
     String *c_classname = Getattr(parent, "name");
     String *symname = Getattr(n, "sym:name");
-    String *declaration = NewStringEmpty();
-    ParmList *l = Getattr(n, "parms");
-    Wrapper *w = NewWrapper();
+    String *declaration;
+    ParmList *l;
+    Wrapper *w;
     String *tm;
     String *wrap_args = NewStringEmpty();
+    String *return_type;
     String *value = Getattr(n, "value");
     String *storage = Getattr(n, "storage");
     bool pure_virtual = false;
@@ -2430,15 +2440,34 @@ done:
       }
     }
 
+    classname = Getattr(parent, "sym:name");
+    type = Getattr(n, "type");
+    name = Getattr(n, "name");
+
+    w = NewWrapper();
+    declaration = NewStringEmpty();
+
     /* determine if the method returns a pointer */
+    decl = Getattr(n, "decl");
     is_pointer = SwigType_ispointer_return(decl);
-    is_void = (Cmp(returntype, "void") == 0 && !is_pointer);
+    is_void = (Cmp(type, "void") == 0 && !is_pointer);
+
+    /* form complete return type */
+    return_type = Copy(type);
+    {
+      SwigType *t = Copy(decl);
+      SwigType *f = SwigType_pop_function(t);
+      SwigType_push(return_type, t);
+      Delete(f);
+      Delete(t);
+    }
 
     /* virtual method definition */
+    l = Getattr(n, "parms");
     String *target;
     String *pclassname = NewStringf("SwigDirector_%s", classname);
     String *qualified_name = NewStringf("%s::%s", pclassname, name);
-    SwigType *rtype = Getattr(n, "conversion_operator") ? 0 : Getattr(n, "classDirectorMethods:type");
+    SwigType *rtype = Getattr(n, "conversion_operator") ? 0 : type;
     target = Swig_method_decl(rtype, decl, qualified_name, l, 0, 0);
     Printf(w->def, "%s", target);
     Delete(qualified_name);
@@ -2461,7 +2490,7 @@ done:
       if (throw_parm_list)
 	Swig_typemap_attach_parms("throws", throw_parm_list, 0);
       for (p = throw_parm_list; p; p = nextSibling(p)) {
-	if (Getattr(p, "tmap:throws")) {
+	if ((tm = Getattr(p, "tmap:throws"))) {
 	  if (gencomma++) {
 	    Append(w->def, ", ");
 	    Append(declaration, ", ");
@@ -2488,7 +2517,7 @@ done:
      */
     if (!is_void) {
       if (!(ignored_method && !pure_virtual)) {
-	String *cres = SwigType_lstr(returntype, "c_result");
+	String *cres = SwigType_lstr(return_type, "c_result");
 	Printf(w->code, "%s;\n", cres);
 	Delete(cres);
       }
@@ -2591,14 +2620,13 @@ done:
       }
 
       if (!idx) {
-	Printf(w->code, "zval **args = NULL;\n");
+	Printf(w->code, "zval **args = NULL;\n", idx);
       } else {
 	Printf(w->code, "zval *args[%d];\n", idx);
       }
       Printf(w->code, "zval *%s, funcname;\n", Swig_cresult_name());
       Printf(w->code, "MAKE_STD_ZVAL(%s);\n", Swig_cresult_name());
-      const char * funcname = GetChar(n, "sym:name");
-      Printf(w->code, "ZVAL_STRINGL(&funcname, (char *)\"%s\", %d, 0);\n", funcname, strlen(funcname));
+      Printf(w->code, "ZVAL_STRING(&funcname, (char *)\"%s\", 0);\n", GetChar(n, "sym:name"));
       Append(w->code, "if (!swig_self) {\n");
       Append(w->code, "  SWIG_PHP_Error(E_ERROR, \"this pointer is NULL\");");
       Append(w->code, "}\n\n");
@@ -2623,7 +2651,16 @@ done:
 
       /* marshal return value */
       if (!is_void) {
+	/* this seems really silly.  the node's type excludes
+	 * qualifier/pointer/reference markers, which have to be retrieved
+	 * from the decl field to construct return_type.  but the typemap
+	 * lookup routine uses the node's type, so we have to swap in and
+	 * out the correct type.  it's not just me, similar silliness also
+	 * occurs in Language::cDeclaration().
+	 */
+	Setattr(n, "type", return_type);
 	tm = Swig_typemap_lookup("directorout", n, Swig_cresult_name(), w);
+	Setattr(n, "type", type);
 	if (tm != 0) {
 	  static const String *amp_result = NewStringf("&%s", Swig_cresult_name());
 	  Replaceall(tm, "$input", amp_result);
@@ -2642,7 +2679,7 @@ done:
 	  Delete(tm);
 	} else {
 	  Swig_warning(WARN_TYPEMAP_DIRECTOROUT_UNDEF, input_file, line_number,
-	      "Unable to use return type %s in director method %s::%s (skipping method).\n", SwigType_str(returntype, 0), SwigType_namestr(c_classname),
+	      "Unable to use return type %s in director method %s::%s (skipping method).\n", SwigType_str(return_type, 0), SwigType_namestr(c_classname),
 	      SwigType_namestr(name));
 	  status = SWIG_ERROR;
 	}
@@ -2669,8 +2706,8 @@ done:
 
     if (!is_void) {
       if (!(ignored_method && !pure_virtual)) {
-	String *rettype = SwigType_str(returntype, 0);
-	if (!SwigType_isreference(returntype)) {
+	String *rettype = SwigType_str(return_type, 0);
+	if (!SwigType_isreference(return_type)) {
 	  Printf(w->code, "return (%s) c_result;\n", rettype);
 	} else {
 	  Printf(w->code, "return (%s) *c_result;\n", rettype);
@@ -2712,6 +2749,7 @@ done:
 
     /* clean up */
     Delete(wrap_args);
+    Delete(return_type);
     Delete(pclassname);
     DelWrapper(w);
     return status;
